#include "DataBlock.h"
#include <stdio.h>
#ifdef WIN32
#include <String.h>
#endif
#include <memory.h>


DataBlock::DataBlock(const unsigned int noBytes):
dataHolderByteSize(noBytes), currentOffset(0), numBytesStored(0)
{
	dataHolder = new char[dataHolderByteSize];
}




DataBlock::DataBlock(const void * data, unsigned int noBytes):
dataHolderByteSize(noBytes), currentOffset(noBytes), numBytesStored(noBytes) {
	dataHolder = new char[dataHolderByteSize];
	memcpy(dataHolder, data, noBytes);
}



DataBlock::~DataBlock(){
	delete [] dataHolder;
}



void DataBlock::write(const void * data, const unsigned int noBytes){
	write(data, noBytes, currentOffset);
}




void DataBlock::write(const void * data, const unsigned int noBytes, const unsigned int startOffset_byte){
	if (startOffset_byte > numBytesStored){
		char message[100];
		sprintf(message,"DataBlock::write() Offset(%d) larger then eof data (%d)",startOffset_byte, numBytesStored);
		throw std::runtime_error(message);
	}

	if (noBytes == 0){
		return;
	}


	if ((startOffset_byte + noBytes) > dataHolderByteSize){
		int newByteSize = startOffset_byte + noBytes  + 64;
		char * newDataHolder = new char[newByteSize];
		memcpy(newDataHolder, dataHolder, dataHolderByteSize);
		delete [] dataHolder;
		dataHolder = newDataHolder;
		dataHolderByteSize = newByteSize;
	}


	memcpy(dataHolder + startOffset_byte, data, noBytes);
	currentOffset = startOffset_byte + noBytes;
	if (currentOffset > numBytesStored){
		numBytesStored = currentOffset;
	}
}





const void * DataBlock::read(const unsigned int noBytes){
	return read(noBytes, currentOffset);
}



const void * DataBlock::read(const unsigned int noBytes, const unsigned int startOffset_byte){
	if ((startOffset_byte + noBytes) > numBytesStored){
		char message[100];
		sprintf(message,"DataBlock::read() Offset(%d) larger then eof data (%d)",startOffset_byte, numBytesStored);
		throw std::runtime_error(message);
	}

	if (noBytes == 0){
		return NULL;
	}

	currentOffset = startOffset_byte + noBytes;
	return dataHolder + startOffset_byte;
}



const char * DataBlock::readString(){
	return readString(currentOffset);
}



const char * DataBlock::readString(const unsigned int startOffset_byte){
	const char * string = readString(dataHolder, numBytesStored, startOffset_byte);
	currentOffset = startOffset_byte + strlen(string) + 1;
	return string;
}




const char * DataBlock::readString(const void * data, const unsigned int dataByteSize, const unsigned int startOffset_byte){
	if (startOffset_byte >= dataByteSize){
		char message[100];
		sprintf(message,"DataBlock::readString() Offset(%d) > eof data (%d)",startOffset_byte, dataByteSize);
		throw std::runtime_error(message);
	}

	bool foundStrEnd = false;
	unsigned int strLen = 0;
	if (data == NULL) {
		char message[100];
		sprintf(message,"DataBlock::readString() DataBlock::readString, Received a NULL data buffer.");
		throw std::runtime_error(message);
	} else {
		//Do nothing.
	}
	for (unsigned int i = startOffset_byte; i < dataByteSize; i++){
		if (((char*)data)[i] == 0){
			strLen = i - startOffset_byte;
			foundStrEnd = true;
			break;
		}
	}
	
	if (!foundStrEnd){
		char message[100];
		sprintf(message,"DataBlock::readString() no null terminated string found at offset %d",startOffset_byte);
		throw std::runtime_error(message);
	}

	return (((char *)data) + startOffset_byte);
}

