#include "SString.h"
#include <stdio.h>
#include <string.h>

char SString::null = 0;


 
SString::SString():stringStore((void *)&null,1)
{
}
	
SString::SString(const std::string &str):stringStore(str.size() + 1)
{
	assign(str.c_str());
}	

SString::SString(const SString & str):stringStore(str.strLen() + 1)
{
	assign(str.getString());
}

SString::SString(const char * str):stringStore(strlen(str) + 1)
{
	assign(str);
}

SString::SString(const char * string, unsigned stringSize):
stringStore((void *) string, stringSize)
{
	stringStore.write((void *)&null,1); // end with null terminator
}

	
SString::~SString(){
	//DINFO(("~SString(): %s destroyed\n", getString()));
}


SString & SString::operator=(const std::string &str){
	assign(str.c_str());
	return *this;
}
	
SString & SString::operator=(const SString & str){
	assign(str.getString());
	return *this;
}

SString & SString::operator=(const char * str){
	assign(str);
	return *this;
}




bool SString::operator==(const std::string &str) const{
	return this->operator==(str.c_str());
}

bool SString::operator==(const SString &str)const{
	return this->operator==(str.getString());
}

bool SString::operator==(const char *str)const{
	return (strcmp(getString(), str) == 0);
}




bool SString::operator!=(const std::string &str) const{
	return this->operator!=(str.c_str());
}

bool SString::operator!=(const SString &str) const{
	return this->operator!=(str.getString());
}

bool SString::operator!=(const char *str) const{
	return (strcmp(getString(), str) != 0);
}




	

SString & SString::operator+=(const std::string &str){
	return append(str.c_str());
}

SString & SString::operator+=(const SString &str){
	return append(str.getString());
}

SString & SString::operator+=(const char *str){
	return append(str);
}


SString SString::operator+(const std::string &str) const{
	SString newStr(*this);
	return newStr+=str;
}

SString SString::operator+(const SString &str) const{
	SString newStr(*this);
	return newStr+=str;
}

SString SString::operator+(const char *str)const {
	SString newStr(*this);
	return newStr+=str;
}

SString SString::operator+(const int val)const{
	char valStr[30];
	sprintf(valStr,"%d",val);
	return *this+valStr;
}

SString SString::operator+(const longlong val)const{
	char valStr[30];
	sprintf(valStr,"%lld",val);
	return *this+valStr;
}

SString SString::operator+(const float val)const{
	char valStr[30];
	sprintf(valStr,"%f",val);
	return *this+valStr;
}

SString SString::operator+(const double val)const{
	char valStr[30];
	sprintf(valStr,"%f",val);
	return *this+valStr;
}




SString& SString::operator<<(const std::string &str){
	(*this)+= str.c_str();
	return *this;
}

SString& SString::operator<<(const SString &str){
	(*this)+= str.getString();
	return *this;
}

SString& SString::operator<<(const char *str){
	(*this)+= str;
	return *this;
}

SString& SString::operator<<(const int val){
	char valStr[30];
	sprintf(valStr,"%d",val);
	(*this)+= valStr;
	return *this;
}

SString& SString::operator<<(const longlong val){
	char valStr[30];
	sprintf(valStr,"%lld",val);
	(*this)+= valStr;
	return *this;
}

SString& SString::operator<<(const float val){
	char valStr[30];
	sprintf(valStr,"%f",val);
	(*this)+= valStr;
	return *this;
}

SString& SString::operator<<(const double val){
	char valStr[30];
	sprintf(valStr,"%f",val);
	(*this)+= valStr;
	return *this;
}






char * SString::getString() const{
	char * str = (char *)stringStore.getDataBlock();
	return str;
}

SString::operator char*() const{
	return getString();
}

SString::operator const char*() const{
	return (const char *)getString();
}
	
SString::operator const std::string() const{
	std::string str(getString());
	  return str;
	}

/**
* not incluing null terminator
*/
unsigned int SString::strLen() const{
	char * str = getString();
	if (str == NULL) return 0;
	return strlen(str);
}


/**
* will return a vector of string separated at c.
* Vector and vector contents are owned by caller.
* empty substrings will NOT be include. 
*/
SArray<SString> * SString::splitAt(char c){
	SArray<SString> * list = new SArray<SString>;
	int startOfStringPos = 0;
	unsigned int currentCharPos = 0;
	char * string = getString();
	while (currentCharPos <= strLen()){
		if ((string[currentCharPos] == c) || (currentCharPos == strLen())){
			int stringSize = currentCharPos - startOfStringPos;
			if ((currentCharPos == strLen()) && (string[currentCharPos] == c)){
				stringSize--;
			}
			if (stringSize > 0){
				SString part(string+startOfStringPos, stringSize);
				list->add(part);
			}
			startOfStringPos = currentCharPos + 1;
		}
		
		currentCharPos++;
	}
	return list;
} 




void SString::assign(const char * str){
	unsigned int len = strlen(str) + 1;
	stringStore.write(str,len,0);
}



SString & SString::append(const char * str){
	unsigned int writePos = this->strLen();
	unsigned int len = strlen(str) + 1;
	stringStore.write(str,len,writePos);

	return *this;
}
